MODULE measure_perf

      USE kinds
      IMPLICIT NONE
      SAVE

CONTAINS


   SUBROUTINE mesure_mmul_perf( n )
      !
      USE mp_global,   ONLY: mpime, root, world_comm, ortho_comm, np_ortho, &
                             me_ortho, ortho_comm_id
      USE descriptors, ONLY: descla_init , la_descriptor
      !
      IMPLICIT NONE
      !
      INCLUDE "mpif.h"
      !
      INTEGER, INTENT(IN) :: n
      !
      REAL(DP), ALLOCATABLE :: c(:,:), a(:,:), b(:,:)
      REAL(DP) :: t1, tcan
      INTEGER  :: nr, nc, ierr
      TYPE(la_descriptor) :: desc
      !
      REAL(DP) :: cclock
      EXTERNAL :: cclock
      !
      ! compute the size of the matrix blocks
      CALL descla_init( desc, n, n, np_ortho, me_ortho, ortho_comm, ortho_comm_id )

      nr = desc%nr
      nc = desc%nc
   
      ! allocate local matrix blocks
      ALLOCATE( a( nr, nc ), c( nr, nc ), b( nr, nc ) )
   
      ! initialize matrix
      a = 1.0d0 / DBLE( n )
      b = 1.0d0 / DBLE( n )
   
      ! some MPIs (OpenMPI) the first time they call a collective routine take too much
      ! time to perform initializations, then perform a dummy call to get meaningful time
      CALL sqr_mm_cannon( 'N', 'N', n, 1.0d0, a, nr, b, nr, 0.0d0, c, nr, desc ) 

      CALL MPI_BARRIER( world_comm, IERR )
      t1 = cclock()

      CALL sqr_mm_cannon( 'N', 'N', n, 1.0d0, a, nr, b, nr, 0.0d0, c, nr, desc )
   
      CALL MPI_BARRIER( world_comm, IERR )
      tcan = cclock() - t1

      ! deallocate matrix blocks
      DEALLOCATE( a, c, b )

      IF( mpime == root ) THEN
         !
         WRITE( 6, 120 ) tcan, np_ortho(1) * np_ortho(2), DBLE(n)*DBLE(n)*DBLE(n)*2.0d0/1.d9/tcan
120      FORMAT(3X,'Parallel MatMul Performance using Cannon algorithm  = ', 1F9.5, ' with ', I4, ' procs', ' GFlops = ', F9.2)
         !
      END IF

      RETURN

   END SUBROUTINE mesure_mmul_perf
   


END MODULE measure_perf

!------------------------------------------------------------------------------!

SUBROUTINE get_arg_matsiz( matsiz )
   !
   IMPLICIT NONE
   !
   INTEGER :: matsiz
   !
   INTEGER :: nargs, iiarg
   CHARACTER(LEN=20) :: np
   INTEGER :: iargc
   !
   matsiz = 1000
   nargs = iargc()
   !
   DO iiarg = 1, ( nargs - 1 )
      !
      CALL getarg( iiarg, np )
      !
      IF ( TRIM( np ) == '-matsiz' .OR. TRIM( np ) == '-size' ) THEN
         !
         CALL getarg( ( iiarg + 1 ), np )
         READ( np, * ) matsiz
         !
      END IF
      !
   END DO
   !
   RETURN
END SUBROUTINE

!------------------------------------------------------------------------------!


program perf

  USE mp_global, ONLY : mp_startup, mp_end
  USE measure_perf
  !
  IMPLICIT NONE
  !
  INTEGER :: matsiz = 100
  !
  CALL mp_startup ( )

  CALL get_arg_matsiz( matsiz )

  CALL mesure_mmul_perf( matsiz )
  !
  CALL mp_end()

end program
